{ **************************************************************************** }
{ *                                                                          * }
{ * DONKEY ANGLER (Gakken, LCD Card Game) Simulator                          * }
{ * by Luca "MADrigal" Antignano, November 2000 (revision: August 2021)      * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:        TImage;
    btn_game_menu_background: TImage;

    im_backdrop:              TImage;
    im_backdrop_logo:         TImage;

    // electronic game
    im_background:            TImage;       im_background_screen_top: TImage;

    btn_time_down:            TImage;       btn_time_top:             TImage;
                                            btn_alarm_top:            TImage;
    btn_acl_down:             TImage;       btn_acl_top:              TImage;

    btn_left_down:            TImage;       btn_left_top:             TImage;
    btn_right_down:           TImage;       btn_right_top:            TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (background, alarm, time/score, game a/b, etc.)
    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_sound: TImage;

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage;

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_enemy_1_1: TImage;     im_enemy_2_1: TImage;     im_enemy_3_1: TImage;
    im_enemy_1_2: TImage;     im_enemy_2_2: TImage;     im_enemy_3_2: TImage;
    im_enemy_1_3: TImage;     im_enemy_2_3: TImage;     im_enemy_3_3: TImage;

    im_hero_1_body: TImage;   im_hero_2_body: TImage;   im_hero_3_body: TImage;
    im_hero_1_leg_1: TImage;  im_hero_2_leg_1: TImage;  im_hero_3_leg_1: TImage;
    im_hero_1_leg_2: TImage;  im_hero_2_leg_2: TImage;  im_hero_3_leg_2: TImage;
    im_hero_1_attack: TImage; im_hero_2_attack: TImage; im_hero_3_attack: TImage;

    im_weapon_1: TImage;      im_weapon_2: TImage;      im_weapon_3: TImage;

    im_item_0_0: TImage;
    im_item_1_1: TImage;      im_item_2_1: TImage;      im_item_3_1: TImage;
    im_item_1_1a: TImage;
    im_item_1_1b: TImage;
    im_item_1_2: TImage;      im_item_2_2: TImage;      im_item_3_2: TImage;
                                                        im_item_3_2a: TImage;
                                                        im_item_3_2b: TImage;
    im_item_1_3: TImage;      im_item_2_3: TImage;      im_item_3_3: TImage;
                              im_item_2_4: TImage;      im_item_3_4: TImage;
                                                        im_item_3_5: TImage;

    im_item_got_1: TImage;    im_item_got_2: TImage;    im_item_got_3: TImage;

{ * DEBUG                                                                    * }
    La_debug: TLabel;

{ * TIMERS                                                                   * }
    Timer_Clock:            TTimer;
    Timer_Game:             TTimer;
    Timer_Game_Over:        TTimer;
    Timer_Game_Reprise_Hit: TTimer;
    Timer_Game_Start:       TTimer;
    Timer_Hero_Animation:   TTimer;
    Timer_Miss_Enemy:       TTimer;
    Timer_Miss_Item:        TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
{}//pm_12_controller:   TMenuItem; (not required)
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Hero             (b: boolean);
    procedure Vid_Show_All_Enemy            (b: boolean);
    procedure Vid_Show_All_Item             (b: boolean);
    procedure Vid_Show_All_Weapon           (b: boolean);

    procedure Vid_Show_Hero                 (var p, f: integer; b: boolean);

    procedure Vid_Show_Panel_Miss           (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_left_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_left_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_right_topMouseUp          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    
    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Game_Over;
    procedure Gam_Game_Reprise_After_Miss;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Game_Reprise_HitTimer   (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_Hero_AnimationTimer     (Sender: TObject);
    procedure Timer_Miss_ItemTimer          (Sender: TObject);
    procedure Timer_Miss_EnemyTimer         (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
{}//procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Donkey Angler (Gakken, LCD Card Game)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.00';
  HH2_CONTENT_DATE    = '2017/12/04';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME    = HH2_CONTENT_NAME + ' Simulator';                                // application's name (appears on taskbar and Splash screen)
{}APP_VERSION = 'Version ' + HH2_CONTENT_VERSION + ' * Final *';                // current version (appears on Splash screen)
{}APP_DATE    = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 4th December, 2017'; // release date (appears on Splash screen)

  URL_EMAIL   = 'mailto:' + HH2_CONTENT_EMAIL;                                  // link to the email address
  URL_PAYPAL  = 'https://www.paypal.me/LucaAntignano';                          // link to the Paypal donation page
  URL_WEBSITE = 'http://' + HH2_CONTENT_WEBSITE + '/';                          // link to the website

  REG_KEY     = 'Software\MADrigal\DonkeyAn50';                                 // registry key

  HINT_CLICK  = 'click or press';
  HINT_TIME   = 'to view current time (hold to view alarm time)';
  HINT_ACL    = 'to reset the game';
  HINT_ALARM  = 'button disabled';

{}MENU_ITEMS = 13;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_got,
  pfs_move,
  pfs_start:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time
  aIm_panel_miss:          array[1..3]       of TImage;                         // miss marks panel

  aIm_hero:                array[1..3, 1..4] of TImage;                         // hero sprites. X=column, Y=frame (1=body, 2 and 3=legs, 4=attacking)
  aIm_enemy:               array[1..3, 1..3] of TImage;                         // magnet/crab
  aIm_weapon:              array[1..3]       of TImage;                         // sabre/fishing rod
  aIm_item_got:            array[1..3]       of TImage;                         // items got (incl. sabre/fishing rod)

  aIm_item:                array[1..3, 0..5] of TImage;                         // normal items sprites
  aIm_item_alt:            array[1..3, 1..2] of TImage;                         // items on a different path, these being (1, 1) and (3, 2). X=column, Y=frame

{ * arrays of mixed types * }
  aKeys:                   array[1..4] of record                                // keys management (values)
                                            int:  integer;
                                            word: word;
                                           end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=ACL/1; 1=ACL/2; 2=Alarm; 3=Time; 4/6=Game A/B start; 5/7=Game A/B; 8=Over
  iScore,                                                                       // score=0-1999
  iHiScoreA,
  iHiScoreB,

  iPosHeroX,                                                                    // 1-3
  iFrameHero,                                                                   // 2-3 (matches the hero's animation frame)
  iMiss,                                                                        // 0-3

  iPosEnemy,                                                                    // 1-3

  iMaxItems                                                                     // 1-6 (maximum number of items on screen)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible

  bControls,
  bKeyPressed                                                                   // left/right button only
                          : boolean;

  // Game variables: loops
  iLoopGame,
  iLoopGameEnemy,
  iLoopGameEnemyCounter,
  iLoopGameOver,
  iLoopGameOverCounter,
  iLoopGameOverHiScore,
  iLoopGameStart,
  iLoopMissEnemy,
  iLoopMissItem,
  iLoopMissPosition
                          : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1; aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_enemy[1, 1] := im_enemy_1_1;    aIm_enemy[2, 1] := im_enemy_2_1;    aIm_enemy[3, 1] := im_enemy_3_1;
  aIm_enemy[1, 2] := im_enemy_1_2;    aIm_enemy[2 ,2] := im_enemy_2_2;    aIm_enemy[3, 2] := im_enemy_3_2;
  aIm_enemy[1, 3] := im_enemy_1_3;    aIm_enemy[2, 3] := im_enemy_2_3;    aIm_enemy[3, 3] := im_enemy_3_3;

  aIm_hero[1, 1] := im_hero_1_body;   aIm_hero[2, 1] := im_hero_2_body;   aIm_hero[3, 1] := im_hero_3_body;
  aIm_hero[1, 2] := im_hero_1_leg_1;  aIm_hero[2, 2] := im_hero_2_leg_1;  aIm_hero[3, 2] := im_hero_3_leg_1;
  aIm_hero[1, 3] := im_hero_1_leg_2;  aIm_hero[2, 3] := im_hero_2_leg_2;  aIm_hero[3, 3] := im_hero_3_leg_2;
  aIm_hero[1, 4] := im_hero_1_attack; aIm_hero[2, 4] := im_hero_2_attack; aIm_hero[3, 4] := im_hero_3_attack;

  aIm_weapon[1] := im_weapon_1;       aIm_weapon[2] := im_weapon_2;       aIm_weapon[3] := im_weapon_3;

  aIm_item_got[1] := im_item_got_1;   aIm_item_got[2] := im_item_got_2;   aIm_item_got[3] := im_item_got_3;

  aIm_item[1, 0] := im_item_0_0;      aIm_item[2, 0] := im_item_0_0;      aIm_item[3, 0] := im_item_0_0;
  aIm_item[1, 1] := im_item_1_1;      aIm_item[2, 1] := im_item_2_1;      aIm_item[3, 1] := im_item_3_1;
  aIm_item[1, 2] := im_item_1_2;      aIm_item[2, 2] := im_item_2_2;      aIm_item[3, 2] := im_item_3_2;
  aIm_item[1, 3] := im_item_1_3;{*}   aIm_item[2, 3] := im_item_2_3;      aIm_item[3, 3] := im_item_3_3;
                                      aIm_item[2, 4] := im_item_2_4;{*}   aIm_item[3, 4] := im_item_3_4;
                                                                          aIm_item[3, 5] := im_item_3_5;{*}

  { * : incorporating the im_weapon[x, 2] sprite }
  aIm_item_alt[1, 1] := im_item_1_1a;                                     aIm_item_alt[3, 1] := im_item_3_2a;
  aIm_item_alt[1, 2] := im_item_1_1b;                                     aIm_item_alt[3, 2] := im_item_3_2b;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Time"  (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 6);                              // "ACL"   (default: "5")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 39);                             // "Left"  (default: "LEFT")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 40);                             // "Right" (default: "RIGHT")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 4 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_time_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint  := HINT_ALARM;                                                     // "Alarm"

  btn_left_top.Hint   := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int];                     // "Left"
  btn_right_top.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int];                     // "Right"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}//               pm_12_controller;
{}aMenuItem[12] := pm_13_n3;
{}aMenuItem[13] := pm_14_exit;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bKeyPressed := false;                                                         // prevents hero movement

  Randomize;                                                                    // initializes the random numbers generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_got);   FSOUND_Sample_Free(pfs_move);
  FSOUND_Sample_Free(pfs_start);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_sound.Visible := b;

  im_panel_game_a.Visible := false;
  im_panel_game_b.Visible := false;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  Vid_Show_All_Enemy(b);
  Vid_Show_All_Weapon(b);
  Vid_Show_All_Hero(b);
  Vid_Show_All_Item(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all hero's sprites                                            }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 4 do
      aIm_hero[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Enemy(b: boolean);
{ displays/hides all enemies' sprites (crab/magnet)                            }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 3 do
      aIm_enemy[i, j].Visible:=b;
end;





procedure TForm1.Vid_Show_All_Item(b: boolean);
{ displays/hides all items' sprites (fish/balloons)                            }
  var
    i: integer;
begin
  // items
  for i := 0 to 3 do
    aIm_item[1, i].Visible := b;

  for i := 1 to 4 do
    aIm_item[2, i].Visible := b;

  aIm_item[3, 1].Visible := b;
  aIm_item[3, 2].Visible := false;

  for i := 3 to 5 do
    aIm_item[3, i].Visible := b;

  // items (alt)
  for i := 1 to 2 do
    aIm_item_alt[1, i].Visible := b;

  aIm_item_alt[3, 1].Visible := false;
  aIm_item_alt[3, 2].Visible := b;

  // items (got)
  for i := 1 to 3 do
    aIm_item_got[i].Visible := false;
end;





procedure TForm1.Vid_Show_All_Weapon(b: boolean);
{ displays/hides all weapons' sprites (sabre/fishing rod)                      }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_weapon[i].Visible := b;
end;





procedure TForm1.Vid_Show_Hero(var p, f: integer; b: boolean);
{ displays/hides the hero in "p" position; when visible, displays him with     }
{ left/right legs up/down according to the "f" parameter                       }
  var
    i :integer;
begin
  if b then
    begin
      aIm_hero[p, 1].Visible := true;
      aIm_hero[p, 2].Visible := f = 2;
      aIm_hero[p, 3].Visible := f = 3;
    //aIm_hero[p, 3] is not required (used when item/enemy got only)
    end
  else
    for i := 1 to 4 do
      aIm_hero[p, i].Visible := false;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := i <= p;                                        // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ displays score in image format                                               }
  var
    i, t: integer;
begin
  t := p;

  while t >= 2000 do                                                            // displays "2000" as "...0"
    Dec(t, 2000);

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      if t >  999 then aIm_panel_digit_pos[1].Picture := aIm_panel_number[t div 1000 mod 10].picture else aIm_panel_digit_pos[1].Picture := nil;                         // thousands
      if t >   99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div  100 mod 10].picture else aIm_panel_digit_pos[2].Picture := aIm_panel_number[0].picture; // hundreds
      if t >    9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div   10 mod 10].picture else aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].picture; // tens
                       aIm_panel_digit_pos[4].Picture := aIm_panel_number[t          mod 10].picture;                                                                    // units
    end
  else                                                                            // hide
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_left_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_left_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_right_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_right_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.FormKeyDown(Sender:TObject; var Key:Word; Shift:TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_time_down   else                              // Time
  if Key = aKeys[2].word then Inp_acl_down    else                              // ACL
  if Key = aKeys[3].word then Inp_left_down   else                              // left
  if Key = aKeys[4].word then Inp_right_down;                                   // right
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_time_up   else                                // Time
  if Key = aKeys[2].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[3].word then Inp_left_up   else                                // left
  if Key = aKeys[4].word then Inp_right_up;                                     // right
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  case iMode of
    1,                                                                          // ACL/2     or
    3,                                                                          // time      or
    8: begin                                                                    // game over ->
      iMode := 2; Gam_Set_Mode;                                                   //         -> alarm
    end;
  end; // case..of
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 2 then                                                             // alarm ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  if iMode <> 0 then                                                            // any mode (and iMode is not 0 already) ->
    begin
      iMode := 0; Gam_Set_Mode;                                                   //                                     -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 0 then                                                             // ACL/1 ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_left_down;
{ Left button: PRESS                                                           }
begin
  btn_left_down.Visible := true;

  case iMode of
    3,                                                                          // time      or
    8: begin                                                                    // game over ->
         bKeyPressed := true;                                                     //         -> game A

         iMode := 5; Gam_Set_Mode;
       end;
    5,                                                                          // game A or
    7: if bControls and (not bKeyPressed) and (iPosHeroX > 1) then              // game B ->
         begin                                                                    //      -> move
           bKeyPressed := true;

           Vid_Show_Hero(iPosHeroX, iFrameHero, false);
           Dec(iPosHeroX);
           Vid_Show_Hero(iPosHeroX, iFrameHero, true);
         end;
  end; // case..of
end;





procedure TForm1.Inp_left_up;
{ Left button: RELEASE                                                         }
begin
  btn_left_down.Visible := false;

  bKeyPressed := false;
end;





procedure TForm1.Inp_right_down;
{ Right button: PRESS                                                          }
begin
  btn_right_down.Visible := true;


  case iMode of
    3,                                                                          // time      or
    8: begin                                                                    // game over ->
         bKeyPressed := true;                                                     //         -> game B

         iMode := 7; Gam_Set_Mode;
       end;
    5,                                                                          // game A or
    7: if bControls and (not bKeyPressed) and (iPosHeroX < 3) then              // game B ->
         begin                                                                  //      -> move left
           bKeyPressed := true;

           Vid_Show_Hero(iPosHeroX, iFrameHero, false);
           Inc(iPosHeroX);
           Vid_Show_Hero(iPosHeroX, iFrameHero, true);
         end;
   end; // case..of
end;





procedure TForm1.Inp_right_up;
{ Right button: RELEASE                                                        }
begin
  btn_right_down.Visible := false;

  bKeyPressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Clock.Enabled            := false;
  Timer_Game.Enabled             := false;
  Timer_Game_Over.Enabled        := false;
  Timer_Game_Reprise_Hit.Enabled := false;
  Timer_Game_Start.Enabled       := false;
  Timer_Hero_Animation.Enabled   := false;
  Timer_Miss_Enemy.Enabled       := false;
  Timer_Miss_Item.Enabled        := false;
end;





procedure TForm1.Gam_Game_Over;
{ records high scores and sets the mode to "game over"                         }
begin
  bControls := false;

  // records high scores into the system registry, if new record has been established
  if (iMode = 5) and (iScore > iHiScoreA) then
    begin
      iHiScoreA := iScore;
      iReg.WriteInteger('', 'HiScore1', iHiScoreA);
    end;

  if (iMode = 7) and (iScore > iHiScoreB) then
    begin
      iHiScoreB := iScore;
      iReg.WriteInteger('', 'HiScore2', iHiScoreB);
    end;

  // sets the variables for the "score/hiscore" blink timer routine
  case iMode of
    5: iLoopGameOverHiScore := iHiScoreA;
    7: iLoopGameOverHiScore := iHiScoreB;
  end;

  iLoopGameOver := 1;
  iLoopGameOverCounter := 1;
  Timer_Game_Over.Interval := 1;
  Timer_Game_Over.Enabled := true;

  // sets mode to "game over"
  iMode := 8;
end;





procedure TForm1.Gam_Game_Reprise_After_Miss;
{ game reprise after a miss has occurred                                       }
begin
  iLoopGameEnemy := 0;
  iLoopGameEnemyCounter := 0;

  Gam_Set_Difficulty;
  Timer_Game.Enabled := true;

  Timer_Hero_Animation.Interval := 250;
  Timer_Hero_Animation.Enabled := true;
end;





procedure TForm1.Gam_Set_Difficulty;
{ calculates difficulty increase, depending on current score                   }
  var
    i, j,
    iHundreds: integer;
begin
  i := iScore;
  iHundreds := 0;

  // calculates how many hundreds in iScore
  while i > 99 do
    begin
      Dec(i, 100);
      Inc(iHundreds);
    end;

  // sets random number for items generator (PRESUMED!)
  case i of
      0..3: iMaxItems := 1;
      4..9: iMaxItems := 2;
    10..19: iMaxItems := 3;
    20..49: iMaxItems := 4;
    50..79: iMaxItems := 5;
    80..99: iMaxItems := 6;
  end; // case..of

  // sets game speed (PRESUMED!)
  case iHundreds of
    0..10: j := 120 - (5 * iHundreds);
    else   j := 120 - (4 * iHundreds);
  end; // case..of

  if j < 60 then j := 60;

  Timer_Game.Interval  := j;

// La_debug.Caption := 'imaxenemy=' + inttostr(iMaxItems) + ', interval=' + inttostr(j);     // DEBUG
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/alarm/ACL mode, depending on the value of the iMode variable  }
  var
    i: integer;
begin
  case iMode of
    0: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         Snd_Silence;

         bControls := false;
       end;
    1: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    2: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         Vid_Show_All_Weapon(true);

         im_panel_time_colon.Visible := true;
         im_panel_sound.Visible := true;
         im_panel_time_pm.Visible := true;

         aIm_panel_digit_pos[1].picture := aIm_panel_number[1].picture;
         aIm_panel_digit_pos[2].picture := aIm_panel_number[2].picture;
         aIm_panel_digit_pos[3].picture := aIm_panel_number[0].picture;
         aIm_panel_digit_pos[4].picture := aIm_panel_number[0].picture;
       end;
    3: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         im_panel_time_colon.Visible := true;
         im_panel_sound.Visible := true;

         Vid_Show_All_Weapon(true);

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;
       end;
    5, 7: begin                                                                 // GAME A (Left button down) or GAME B (Right button down)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         im_panel_game_a.Visible := iMode = 5;
         im_panel_game_b.Visible := iMode = 7;

         im_panel_sound.Visible := true;

         iScore := 0;
         iMiss := 0;

         case iMode of
           5: Vid_Show_Panel_Score(iHiScoreA, true);
           7: Vid_Show_Panel_Score(iHiScoreB, true);
         end; // case..of

         Vid_Show_All_Weapon(true);

         Gam_Set_Difficulty;

         iPosHeroX := 2;
         iFrameHero := 3;
         Vid_Show_Hero(iPosHeroX, iFrameHero, true);

         iLoopGameStart := 1;
         Timer_Game_Start.Interval := 1;
         Timer_Game_Start.Enabled := true;
       end;
    8: begin
         // see "Gam_Game_Over"
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i,
    j,
    tmpAction,
    tmpItemsOnScreen: integer;

    tmpGenerate,
    tmpItem1: boolean;
begin
  tmpAction := 0; // 0: no collisions; 1: item/enemy caught; 2: miss (item); 3: miss (enemy)

  // collision check (hero catches items)
  if aIm_item[iPosHeroX, 2 + iPosHeroX].Visible then
    begin
      tmpAction := 1;

      bControls := false;

      Timer_Game.Enabled := false;
      Timer_Hero_Animation.Enabled := false;

      aIm_item[iPosHeroX, 2 + iPosHeroX].Visible := false;
      aIm_hero[iPosHeroX, 4].Visible := true;
      aIm_item_got[iPosHeroX].Visible := true;

      Inc(iScore);
      Vid_Show_Panel_Score(iScore, true);
      Gam_Set_Difficulty;

      Snd_Play(pfs_got);

      Timer_Game_Reprise_Hit.Interval := 440;
      Timer_Game_Reprise_Hit.Enabled := true;
    end;

  // collision check (hero catches enemy)
  if aIm_enemy[iPosHeroX, 1].Visible then
    begin
      tmpAction := 1;
      iLoopGameEnemy := 1;
      iLoopGameEnemyCounter := 0;

      bControls := false;

      Timer_Game.Enabled := false;
      Timer_Hero_Animation.Enabled := false;

      aIm_weapon[iPosHeroX].Visible := false;

      for i := 1 to 2 do
        aIm_enemy[iPosHeroX, i].Visible := false;

      aIm_hero[iPosHeroX, 4].Visible := true;

      Inc(iScore, 2);
      Vid_Show_Panel_Score(iScore, true);
      Gam_Set_Difficulty;

      Snd_Play(pfs_got);

      Timer_Game_Reprise_Hit.Interval := 440;
      Timer_Game_Reprise_Hit.Enabled := true;
    end;

  // case: no "hero catches something" collisions (part 1)
  if tmpAction = 0 then
    begin
      // collision check (misses an item)
      for i := 1 to 3 do
        if aIm_item[i, i + 2].Visible and (i <> iPosHeroX) then
          begin
            tmpAction := 2;

            Gam_Stop_All_Timers;
            Snd_Silence;

            iLoopMissPosition := i;

            iLoopMissItem := 1;
            Timer_Miss_Item.Interval := 200;
            Timer_Miss_Item.Enabled := true;
          end;

      // collision check (misses an enemy)
      if (iMode = 7) and (((iLoopGame = 1) or (iLoopGame = 3)) or (iLoopGame = 5)) then
        begin
          // enemy management (collision)
          if iLoopGameEnemyCounter > 0 then
            Inc(iLoopGameEnemyCounter);

          if iLoopGameEnemyCounter = 4 then                                     // after 4 ticks
            begin
              tmpAction := 3;                                                     // a miss has occurred

              Gam_Stop_All_Timers;
              Snd_Silence;

              iLoopMissEnemy := 1;
              Timer_Miss_Enemy.Interval := 200;
              Timer_Miss_Enemy.Enabled := true;
            end;

          // enemy management (generation and visualisation)
          Inc(iLoopGameEnemy);

          if iLoopGameEnemy = 6 then                                            // at every 6 ticks
            begin
              if Random(4) = 0 then                                               // Random(4)
                begin                                                               // generates a new enemy
                  iPosEnemy := Random(3) + 1;

                  for i := 1 to 2 do
                    aIm_enemy[iPosEnemy, i].Visible := true;

                  iLoopGameEnemyCounter := 1;
                end
              else                                                                  // or deletes on-screen enemies, if existing
                iLoopGameEnemyCounter := 0;

              // resets loop
              iLoopGameEnemy := 1
            end;
        end; // (if (iMode = 7) and (((iLoopGame = 1) or (iLoopGame = 3)) or (iLoopGame = 5)))
    end; // (if tmpAction = 0)

  // case: no "hero catches something" collisions (part 2)
  if tmpAction = 0 then
    begin
      // items movement
      case iLoopGame of
        2: case iMode of
             5: begin
                  for i := 3 downto 1 do
                    aIm_item[1, i].Visible := aIm_item[1, i - 1].Visible;

                  aIm_item[1, 0].Visible := false;

                  if aIm_item[1, 3].Visible then
                    aIm_weapon[1].Visible := false;

                  // audio
                  if aIm_item[1, 2].Visible then
                    Snd_Play(pfs_move);
                end;
             7: begin
                  tmpItem1 := false;

                  aIm_item[1, 3].Visible := aIm_item[1, 2].Visible;

                  if aIm_item_alt[1, 2].Visible then
                    begin
                      aIm_item_alt[1, 2].Visible := false;
                      tmpItem1 := true;
                    end;

                  if aIm_item_alt[1, 1].Visible then
                    begin
                      aIm_item_alt[1, 1].Visible := false;
                      aIm_item_alt[1, 2].Visible := true;
                    end;

                  if random(7) < 2 then
                    begin
                      aIm_item_alt[1, 1].Visible := aIm_item[1, 1].Visible;
                      aIm_item[1, 1].Visible := false;
                    end;

                  for i := 2 downto 1 do
                    aIm_item[1, i].Visible := aIm_item[1, i - 1].Visible;

                  if tmpItem1 then
                    aIm_item[1, 1].Visible := true;

                  aIm_item[1, 0].Visible := false;

                  if aIm_item[1, 3].Visible then
                    aIm_weapon[1].Visible := false;

                  // audio
                  if aIm_item[1, 2].Visible or aIm_item_alt[1, 1].Visible then
                    Snd_Play(pfs_move);
                end;
           end; // case..of
        4: begin
             for i := 4 downto 1 do
               aIm_item[2, i].Visible := aIm_item[2, i - 1].Visible;

             aIm_item[2, 0].Visible := false;

             if aIm_item[2, 4].Visible then
               aIm_weapon[2].Visible := false;

             // audio
             if aIm_item[2, 3].Visible then
               Snd_Play(pfs_move);
           end;

        6: case iMode of
             5: begin
                  for i := 5 downto 1 do
                    aIm_item[3, i].Visible := aIm_item[3, i - 1].Visible;

                  aIm_item[3, 0].Visible := false;

                  if aIm_item[3, 5].Visible then
                    aIm_weapon[3].Visible := false;

                  // audio
                  if aIm_item[3, 4].Visible then
                    Snd_Play(pfs_move);
                end;
             7: begin
                  tmpItem1 := false;

                  for i := 5 downto 4 do
                    aIm_item[3, i].Visible := aIm_item[3, i - 1].Visible;

                  if aIm_item_alt[3, 2].Visible then
                    begin
                      aIm_item_alt[3, 2].Visible := false;
                      tmpItem1 := true;
                    end;

                  if aIm_item_alt[3, 1].Visible then
                    begin
                      aIm_item_alt[3, 1].Visible := false;
                      aIm_item_alt[3, 2].Visible := true;
                    end;

                  if random(7) < 2 then
                    begin
                      aIm_item_alt[3, 1].Visible := aIm_item[3, 2].Visible;
                      aIm_item[3, 2].Visible := false;
                    end;

                  for i := 3 downto 1 do
                    aIm_item[3, i].Visible := aIm_item[3, i - 1].Visible;

                  if tmpItem1 then
                    aIm_item[3, 2].Visible := true;

                  aIm_item[3, 0].Visible := false;

                  if aIm_item[3, 5].Visible then
                    aIm_weapon[3].Visible := false;

                  // audio
                  if aIm_item[3, 4].Visible then
                    Snd_Play(pfs_move);
                end;
             end; // case..of
      end; // case..of

      // items generation
      tmpGenerate := false;

      // first set of rules
      if ((iLoopGame = 1) or (iLoopGame = 3)) or (iLoopGame = 5) and (Random(3) = 0) then tmpGenerate := true;
      if (iMode = 7) and (iLoopGame = 1) and aIm_item_alt[1, 2].Visible then tmpGenerate := false;
      if (iMode = 7) and (iLoopGame = 5) and aIm_item_alt[3, 1].Visible then tmpGenerate := false;
      if (iScore < 41) and ((iLoopGame = 1) or ((iLoopGame = 3) or (iLoopGame = 5))) and aIm_item[trunc((iLoopGame + 1) / 2), 1].Visible then tmpGenerate := false; // 2 items in a row
      if ((iLoopGame = 1) or ((iLoopGame = 3) or (iLoopGame = 5))) and (aIm_item[trunc((iLoopGame + 1) / 2), 1].Visible and aIm_item[trunc((iLoopGame + 1 )/ 2), 2].Visible) then tmpGenerate := false; // 3 items in a row

      // second set of rules (items on screen)
      tmpItemsOnScreen := 0;

      for i := 1 to 3 do for j := 1 to i + 2 do if aIm_item[i, j].Visible then Inc(tmpItemsOnScreen);
      for i := 1 to 2 do if aIm_item_alt[1, i].Visible or aIm_item_alt[3, i].Visible then Inc(tmpItemsOnScreen);

      if tmpItemsOnScreen >= iMaxItems then tmpGenerate := false;
      if tmpItemsOnScreen = 0 then tmpGenerate := true;

      // actual generation
      if tmpGenerate then
        aIm_item[(trunc(iLoopGame + 1)div 2), 0].Visible := true; // 2, 4 or 6

      // loops action
      Inc(iLoopGame);

      if iLoopGame = 7 then
        iLoopGame := 1;
    end; // (if tmpAction = 0)
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ score and high score blinks for 5 minutes, then mode is set back to "time"   }
  var
    i: integer;
begin
  // displays score and high score alternatively
  case iLoopGameOver of
    1, 3: begin
         i := 0; Vid_Show_Panel_Score(i, false);

         Timer_Game_Over.Interval := 100;
       end;
    2: begin
         Vid_Show_Panel_Score(iLoopGameOverHiScore, true);

         Timer_Game_Over.Interval := 350;
       end;
    4: begin
         Vid_Show_Panel_Score(iScore, true);

         Timer_Game_Over.Interval := 350;
       end;
  end; // case..of

  // loops action (1)
  Inc(iLoopGameOver);

  if iLoopGameOver = 5 then
    iLoopGameOver := 1;

  // loos action (2)
  Inc(iLoopGameOverCounter);

  // reverts back to "time" mode after 2 minutes
  if iLoopGameOverCounter = 133 then // 120.000/(100 + 350 + 100 + 350) ms = 120.000/900 ms = 133 ticks
    begin
      iMode := 3;
      Gam_Set_Mode;

      Timer_Game_Over.Enabled := false;
    end;
end;





procedure TForm1.Timer_Game_Reprise_HitTimer(Sender: TObject);
{ action reprise after successful hit with item/enemy                          }
begin
  aIm_hero[iPosHeroX, 4].Visible := false;

  aIm_item_got[iPosHeroX].Visible := false;
  aIm_weapon[iPosHeroX].Visible := true;

  bControls := true;

  Gam_Set_Difficulty;
  Timer_Game.Enabled := true;

  Timer_Hero_Animation.Interval := 250;
  Timer_Hero_Animation.Enabled := true;

  Timer_Game_Reprise_Hit.Enabled := false;
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ game start initialisation                                                    }
begin
  case iLoopGameStart of
    1: begin
         Snd_Play(pfs_start);

         Timer_Game_Start.Interval := 150;
       end;
    2: begin
         // game begins
         iLoopGameEnemy := 1;
         iLoopGameEnemyCounter := 0;
         iLoopGame := 1;
      // Gam_Set_Difficulty; {not needed, done already}
         Timer_Game.Enabled := true;

         Timer_Hero_Animation.Interval := 250;
         Timer_Hero_Animation.Enabled := true;

         bControls := true;

         Timer_Game_Start.Interval := 900;
       end;
    3: begin
         // displays score (zero)
         Vid_Show_Panel_Score(iScore, true);

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_Hero_AnimationTimer(Sender: TObject);
{ animation of the hero's sprite                                               }
  var
    i: integer;
begin
  // loops action between 2 and 3
  Inc(iFrameHero);

  if iFrameHero = 4 then
    iFrameHero := 2;

  // visualisation
  Vid_Show_Hero(iPosHeroX, iFrameHero, true);
end;





procedure TForm1.Timer_Miss_EnemyTimer(Sender: TObject);
{ a miss has occurred: an enemy has hit                                        }
begin
  // audio and visual animation
  case iLoopMissEnemy of
    1..7: begin
         aIm_enemy[iPosEnemy, 2].Visible := not Odd(iLoopMissEnemy);
         aIm_enemy[iPosEnemy, 3].Visible := Odd(iLoopMissEnemy);

         Snd_Play(pfs_move);
       end;
  end; // case..of

  // special cases
  case iLoopMissEnemy of
    1: Timer_Miss_Enemy.Interval := 290;
    7: begin
         Inc(iMiss);
         Vid_Show_Panel_Miss(iMiss);
       end;
    8: begin
         Vid_Show_All_Enemy(false);

         if iMiss < 3 then
           Gam_Game_Reprise_After_Miss // game reprise
         else
           Gam_Game_Over; // game over

         Timer_Miss_Enemy.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMissEnemy);
end;





procedure TForm1.Timer_Miss_ItemTimer(Sender: TObject);
{ a miss has occurred: an item has been missed                                 }
begin
  // audio and visual animation
  case iLoopMissItem of
    1..7: begin
         aIm_item[iLoopMissPosition, 2 + iLoopMissPosition].Visible := not Odd(iLoopMissItem);

         Snd_Play(pfs_move);
       end;
  end; // case..of

  // special cases
  case iLoopMissItem of
    1: Timer_Miss_Item.Interval := 290;
    7: begin
         Inc(iMiss);
         Vid_Show_Panel_Miss(iMiss);
       end;
    8: begin
         aIm_weapon[iLoopMissPosition].Visible := true;

         if iMiss < 3 then
           Gam_Game_Reprise_After_Miss // game reprise
         else
           Gam_Game_Over; // game over

        Timer_Miss_Item.Enabled := false;
       end;
  end; // case..of

  // loops action
  Inc(iLoopMissItem);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Height := 16;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
